
/*
 * get_data.c - get data (image header and body)
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include "sentry.h"

#include <errno.h>

char *buff = NULL;

/*
 * get_data - whenever data packets are arrived from server,
 *            this procedure will be called.
 */
int
get_data(int datafd)
{
  struct sockaddr_in from;
  int fromlen;
  char packet[8192];
  int nread;
  sentry_image_header *hdr;
  int chk;
  int image_format, image_size, image_width, image_height;

  /*
   * receive packets 
   */
  while (1) {
    fromlen = sizeof(from);
    /* bzero(packet, sizeof(packet));*/

    if (debug) {
      time_t now;
      time(&now);
      fprintf(stderr, "waiting for packets... %d\n", now);
    }
    nread = recvfrom(datafd, packet, sizeof(packet), 0, (struct sockaddr *)&from, &fromlen);
    
    if (nread < 0) {
      if (errno == EINTR) 
	continue;

      fprintf(stderr, "get_data: fail recvfrom(): %s\n", sys_errlist[errno]);
      return 1;
    }
    
    /*
     * header check
     */
    if ((nread -= SENTRY_IMAGE_HEADER_LEN) < 0) {
      fprintf(stderr, "get_data: header length is wrong.\n");
      return 1;
    }

    hdr = (sentry_image_header *)packet;
    
    if (hdr->hd_ver != SENTRY_VERSION) {
      fprintf(stderr, "get_data: header version is wrong.\n");
      return 1;
    }

    chk = udp_reassemble(hdr, &packet[SENTRY_IMAGE_HEADER_LEN], nread, &buff);
    if (chk > 0) break;

    if (chk == 0) {
      /* need more udp packets, to construct image frame */
      fprintf(stdout, ".");
      continue;
      return 0;
    } 
    else /* chk < 0 */
      return 1;

  } /* end-of-while */
    
  /* 
   * check timestamp ?
   */
  /* no-op */
  
  image_format = ntohl(hdr->format);
  image_size = ntohl(hdr->size);
  image_width = ntohl(hdr->width);
  image_height = ntohl(hdr->height);
  
  /*
   * save image to shared memory as it is, 
   *   reagardless of image format.
   */
  save_thru(image_size, image_format, &buff);
  
  return;
}
