
/*
 * request.c - send QoS request to server
 */

#include <stdio.h>
#include <sys/time.h>
#include <signal.h>

#include "sentry.h"

extern int debug;
extern int verbose;

extern int ctrlfd;
extern int hivision_flag;

extern u_char client_type;
extern int server_zoom;
extern int server_size;
extern int server_color;
extern int server_format;
extern int server_rate;

extern int request_interval;

sentry_request_hdr req_hdr;



void init_request();
void request_call();

void
init_request()
{
  struct itimerval itval;

  itval.it_value.tv_sec = request_interval;
  itval.it_value.tv_usec = 0;
  itval.it_interval.tv_sec = request_interval;
  itval.it_interval.tv_usec = 0;

  signal(SIGALRM, request_call);
  setitimer(ITIMER_REAL, &itval, NULL);
}

void
request_call()
{
  if (ctrlfd < 0)
    exit (1);

  /*
   * I don't know it is proper to to change to hivision size here. :)
   */
  if (hivision_flag) {
    switch(server_size) {
    case SIZE_640x480:
      server_size = SIZE_640x360;
      break;
    case SIZE_320x240:
      server_size = SIZE_320x180;
      break;
    case SIZE_212x160:
      server_size = SIZE_212x120;
      break;
    case SIZE_160x90:
      server_size = SIZE_160x90;
      break;
    }
  }

  if (verbose) 
    fprintf(stderr, "request_call: type# %x, size# %d, zoom# %d, color# %d, format# %d, rate# %d\n", 
	    client_type, server_size, server_zoom, server_color, server_format, server_rate);

  req_hdr.hd_ver = SENTRY_VERSION;
  req_hdr.type = client_type;
  req_hdr.size = htonl(server_size);
  req_hdr.zoom = htonl(server_zoom);
  req_hdr.color = htonl(server_color);
  req_hdr.format = htonl(server_format);
  req_hdr.rate = htonl(server_rate);

  write(ctrlfd, (char *)&req_hdr, sizeof(req_hdr));
}


