/*
 * Sentry Video System
 *   is video server with QoS control from clients' requests.
 *
 *      original by akira-y@is.aist-nara.ac.jp, 1994,1995
 *      modified by k-chinen@is.aist-nara.ac.jp, 1996
 *      modified by satosi-f@is.aist-nara.ac.jp, 1997,2000
 *  
 *  $Id: $
 */

/*
 * get_request.c - get request
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "sentry.h"
#include "sentryd.h"

#include <netinet/in.h>
#include <sys/time.h>
#include <signal.h>
#include <errno.h>

extern int errno;
extern int debug;
extern int verb;

char recvbuf[8192];
struct itimerval sendtimer;

#if 0
struct {
  union {
    int val[QOS_PARAMS];
    struct {
      int size;
      int zoom;
      int color;
      int format;
      int rate;
    } qos_val;
  } attr;
  u_char change_flag;
} req_val;
#endif

#if 0
struct {
  int val;              /* qos value */
  int change_flag;      /* change flag */
} req_val[QOS_PARAMS];
#endif

void change_video_qos();
void change_network_qos();
void set_rate();
void set_qos();

int 
get_udp_request(int sockfd)
{
  int nr;
  struct sockaddr_in from;
  int fromlen;
  sentry_request_hdr *sentry_req_hdr;
  u_char type;
  int dataport;
  extern char *vserver;

again:
  nr = recvfrom(sockfd, recvbuf, sizeof(recvbuf), 0, (struct sockaddr *)&from, &fromlen);
  if (nr <= 0) {
    if (errno == EINTR)
      goto again;
    fprintf(stderr, "get_udp_request: recvfrom: %s.\n", sys_errlist[errno]);
    return -1;
  }
  
  /* header check */
  sentry_req_hdr = (sentry_request_hdr *)recvbuf;
  if (sentry_req_hdr->hd_ver != SENTRY_VERSION) {
    fprintf(stderr, "get_udp_request: invalid protocol# %d.\n", sentry_req_hdr->hd_ver);
    return -1;
  }
    
  type = sentry_req_hdr->type;
  /* switch(type) {*/
  while (type) {
    if (type == OPEN_UDP_UNICAST ||
	type == OPEN_UDP_MULTICAST) {
      /*
	case OPEN_UDP_UNICAST:
	case OPEN_UDP_MULTICAST:
	*/
      dataport = ntohl((int)sentry_req_hdr->port);
      if (add_client(type, &from, dataport) < 0) {
	fprintf(stderr, "sentryd: cannot add a new client.\n");
	return -1;
      }
      
      if (!check_video(vserver))
	init_video(vserver);
      
      set_qos(QOS_SIZE, ntohl(sentry_req_hdr->size));
      set_qos(QOS_ZOOM, ntohl(sentry_req_hdr->zoom));
      set_qos(QOS_COLOR, ntohl(sentry_req_hdr->color));;
      set_qos(QOS_FORMAT, ntohl(sentry_req_hdr->format));
      /* change_video_qos(); */
      
      set_qos(QOS_RATE, ntohl(sentry_req_hdr->rate));
      /* change_network_qos(); */

      break;
#if 0
      qos_param[QOS_SIZE]   = ntohl(sentry_req_hdr->size);
      qos_param[QOS_ZOOM]   = ntohl(sentry_req_hdr->zoom);
      qos_param[QOS_COLOR]  = ntohl(sentry_req_hdr->color);
      qos_param[QOS_FORMAT] = ntohl(sentry_req_hdr->format);
      change_video_qos();
      
      qos_param[QOS_RATE]   =  ntohl(sentry_req_hdr->rate);
      change_network_qos();
#endif
    }
      
    /* case OPEN_TCP: */
    if (type == OPEN_TCP) {
      fprintf(stderr, "sentryd: OPEN_TCP request for udp port is rejected.\n");
      return -1;
    }
      
    /* case CLOSE: */
    if (type == CLOSE) {
      clear_client(find_client(&from));
      return 0;
    }

    /*  case QOS_REQUEST_SIZE: */
    if (type & QOS_REQUEST_SIZE) {
      set_qos(QOS_SIZE, ntohl(sentry_req_hdr->size));
      type &= ~QOS_REQUEST_SIZE;
    }

    /* case QOS_REQUEST_ZOOM: */
    if (type & QOS_REQUEST_ZOOM) {
      set_qos(QOS_ZOOM, ntohl(sentry_req_hdr->zoom));
      type &= ~QOS_REQUEST_ZOOM;
    }

    /*  case QOS_REQUEST_COLOR: */
    if (type & QOS_REQUEST_COLOR) {
      set_qos(QOS_COLOR, ntohl(sentry_req_hdr->color));
      type &= ~QOS_REQUEST_COLOR;
    }
    
    /*  case QOS_REQUEST_FORMAT: */
    if (type & QOS_REQUEST_FORMAT) {
      set_qos(QOS_FORMAT, ntohl(sentry_req_hdr->format));
      type &= ~QOS_REQUEST_FORMAT;
    }
    
    /* case QOS_REQUEST_RATE: */
    if (type & QOS_REQUEST_RATE) {
      set_qos(QOS_RATE, ntohl(sentry_req_hdr->rate));
      type &= ~QOS_REQUEST_RATE;
    }
    
    /* default: */
    else {
      fprintf(stderr, "sentryd: unknown request type. ignored.\n");
      break;
    }
  } /* end-of "type check" */
  
  change_video_qos();
  change_network_qos();
}
  
/*
 * QoS request control 
 */
static u_char nqflag = QOS_REQUEST_RATE;

void
change_network_qos() 
{

  if (req_val.change_flag & nqflag)
    set_rate();
}

void
set_rate()
{
  int sec, usec;
  int interval;
  struct timeval tv;
  extern void send_image();

  signal(SIGALRM, SIG_IGN);

  interval = 100 * 1000000 / GET_QOS(QOS_RATE);
  sec = interval / 1000000;
  usec = interval % 1000000;

  sendtimer.it_interval.tv_sec = sec;
  sendtimer.it_interval.tv_usec = usec;
  sendtimer.it_value.tv_sec = sec;
  sendtimer.it_value.tv_usec = usec;

  signal(SIGALRM, send_image);
  setitimer(ITIMER_REAL, &sendtimer, NULL);

  CLR_CHANGE_FLAG(QOS_RATE);
}

void
set_qos(int type, int val)
{
  int oldval;

  if (type < 0 || type >= QOS_PARAMS) 
    return;

  if (req_val.attr.val[type] != val) {
    oldval = val;
    req_val.attr.val[type] = val;
    req_val.change_flag |= (0x01 << type);

    if (verb)
      fprintf(stderr, "change QoS params# %d, %d -> %d\n", 
	      type, oldval, req_val.attr.val[type]);
  }
}

void 
init_req_val(void)
{
  int type;

  for(type=0; type<QOS_PARAMS; type++) {
    req_val.attr.val[type] = 0;
    if (verb)
      fprintf(stderr, "init_req_val: type# %d initialized.\n", type);
  }
  req_val.change_flag = 0;
}

