/*
 * Sentry Video System
 *   is video server with QoS control from clients' requests.
 *
 *      original by akira-y@is.aist-nara.ac.jp, 1994,1995
 *      modified by k-chinen@is.aist-nara.ac.jp, 1996
 *      modified by satosi-f@is.aist-nara.ac.jp, 1997,2000
 *  
 *  $Id: $
 */

/*
 * manage_client.c - manage (register, release, etc) clients.
 */

#include <stdio.h>
#include <sys/types.h>

#include "sentry.h"

#include <netinet/in.h>
#include <sys/socket.h>
#include <errno.h>

#define MAX_CLIENTS     64
struct q_client {
  int id;
#define INVALID        (-1)
#define UDP_UNICAST    0
#define UDP_MULTICAST  1
#define TCP            2
  int flag;                   /* flag */
  int sockfd;                   /* send socket fd */
  struct sockaddr_in addr;      /* family/address/port */
  int rate;                     /* request rate */
  struct q_client *next;
#if 0
  struct q_client *prev;        
#endif
} client_pool[MAX_CLIENTS];
  
struct q_client *free;
struct q_client *clients_udp_uni;
struct q_client *clients_udp_multi;
struct q_client *clients_tcp;

#ifdef SUPPORT_MULTIRATE
struct {
  int rate_interval;
  int rate_count;
} client_rate[MAX_CLIENTS];
#endif

extern int debug;
extern int verb;





int
add_client(u_char type, struct sockaddr_in *addr, int fd) 
{
  int id;
  struct q_clinet *cli, *cli_t;

  /* client address check */
  if (addr->sin_family != AF_INET) {
    fprintf(stderr, "sentryd: clients %s:%d address family is invalid.\n", 
	    inet_ntoa(addr->sin_addr), ntohs(addr->sin_port));
    return -1;
  }

  if ((id = find_client(addr)) < 0) {
    /* new client */
    if ((id = malloc_client()) < 0)
      return id; /* return -1 */

    switch(type) {
    case OPEN_UDP_UNICAST:
      client_pool[id].flag = UDP_UNICAST;
      client_pool[id].next = clients_udp_uni;
      clients_udp_uni = &client_pool[id];
      break;
    case OPEN_UDP_MULTICAST:
      client_pool[id].flag = UDP_MULTICAST;
      client_pool[id].next = clients_udp_multi;
      clients_udp_multi = &client_pool[id];
      break;
    default:
      return -1;
    }

    client_pool[id].sockfd = fd;
    client_pool[id].addr.sin_family = addr->sin_family;
    client_pool[id].addr.sin_addr.s_addr = ntohl(addr->sin_addr.s_addr);
    client_pool[id].addr.sin_port = ntohs(addr->sin_port);

    if (debug) {
      fprintf(stderr, "add_client: sock/port# %d, id# %d, %s:%d, type# %x\n", 
	      client_pool[id].sockfd, id, 
	      inet_ntoa(client_pool[id].addr.sin_addr),
	      client_pool[id].addr.sin_port);
    }
      
  }

  return id;
}

int
find_client(struct sockaddr_in *addr) 
{
  int id;
  for(id=0; id<MAX_CLIENTS; id++) {
    if (!bcmp((char *)addr, (char *)&(client_pool[id].addr), sizeof(struct sockaddr_in)))
      return id;
  }
  return -1;
}

int
malloc_client()
{
  int id;
  for(id=0; id<MAX_CLIENTS; id++) {
    if (client_pool[id].flag == INVALID)
      return id;
  }
  return -1;
    
}



void
clear_client(int id)
{
  int flag;
  struct q_client *cli, *cli_t, *cli_prev;

#ifdef SUPPORT_MULTIRATE
  int cid, maxcid = -1;
  int maxrate = 0;
  extern void set_qos(), set_client_rate();
#endif

  if (id < 0 || id > MAX_CLIENTS) return;

  /* set client flag off */
  switch(client_pool[id].flag) {
  case UDP_UNICAST:
    cli_t = clients_udp_uni;
    break;
  case UDP_MULTICAST:
    cli_t = clients_udp_multi;
    break;
  case TCP:
  default:
    cli_t = clients_tcp;
    break;
  }

  for(cli=cli_t; cli; cli=cli->next) {
      if (cli->id == id)
	if (cli == cli_t)
	  cli_t = cli->next;
	else
	  cli_prev->next = cli->next;
      cli_prev = cli;
  }
  client_pool[id].flag = INVALID;

  /* update free list */
  client_pool[id].next = free;
  free = &client_pool[id];
  
  if (verb) 
    fprintf(stdout, "client[%d] is clear.\n", id);
}

void 
init_client(void)
{
  int i;

  for(i=0; i<MAX_CLIENTS; i++) {
    client_pool[i].id = i;
    client_pool[i].flag = INVALID;
    client_pool[i].next = NULL;

    if (i<MAX_CLIENTS-1) 
      client_pool[i].next = &client_pool[i+1];
    else 
      client_pool[i].next = NULL;
  }    

  free = client_pool;
  clients_udp_uni = NULL;
  clients_udp_multi = NULL;
  clients_tcp = NULL;

  if (verb) {
    struct q_client *cli;
    int id = 0;

    for(cli=free; cli; cli=cli->next)
      fprintf(stdout, "init_client: free[%d]: flag# %u\n", 
	      id++, cli->flag);
  }
}
