/*
 * Sentry Video System
 *   is video server with QoS control from clients' requests.
 *
 *      original by akira-y@is.aist-nara.ac.jp, 1994,1995
 *      modified by k-chinen@is.aist-nara.ac.jp, 1996
 *      modified by satosi-f@is.aist-nara.ac.jp, 1997,2000
 *  
 *  $Id: $
 */

/*
 * send_image.c - video handling procedures.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <errno.h>

#include "sentry.h"
#include "sentryd.h"

extern  fd_set writemask;
extern  int maxwritefds;

int     xsize, ysize;
int     grab_xsize, grab_ysize, xoff, yoff;
int     frameSize;

char   *dataPtr;
char   *new_dataPtr;

void   show_stat();

void
send_image(void)
{
  sentry_image_header *hdr;
  struct timeval tv, waittime;
  int new_xsize, new_ysize;
  int fd;
  int datalen, rest, send_datalen, send_copunt;
  int nwrites;
  char *sendptr;
  int encode_type;

#ifdef USE_IOVEC
  sentry_image_header header;
  struct iovec iov[2];
#else
  char packet[8192];
#endif /* USE_IOVEC */
#ifdef ALIVE
  extern void set_alive_mark();
#endif /* ALIVE */
  extern void set_rate();

  dataPtr = (char *)capture_frame();
  gettimeofday(&tv, NULL);

#ifdef ALIVE
  (void)set_alive_mark();
#endif 

  /*
   * grab image 
   */
  if (xoff > 0 || yoff > 0) {
    if (grab_image(&dataPtr, &frameSize, &xsize, &ysize, grab_xsize, grab_ysize, xoff, yoff) < 0) {
      fprintf(stderr, "send_image: cannot grab image (%dx%d+%d+%d).\n", grab_xsize, grab_ysize, xoff, yoff);
    }
  }

  /*
   * convert video format
   */
  switch(req_val.attr.val[QOS_FORMAT]) {
  case ENCODE_JPEG:
    datalen = convert_jpeg(dataPtr, frameSize, xsize, ysize, &new_xsize, &new_ysize, &new_dataPtr);
    encode_type = ENCODE_JPEG;
    break;

  case ENCODE_NONE:
  default:
    datalen = convert_none(dataPtr, frameSize, xsize, ysize, &new_xsize, &new_ysize, &new_dataPtr);
    encode_type = ENCODE_NONE;
    break;
  }

  /* send image */
#ifdef USE_IOVEC
  hdr = &header;

  hdr->hd_ver = SENTRY_VERSION;
  hdr->ht_ts[0] = htonl(tv.tv_sec);
  hdr->ht_ts[1] = htonl(tv.tv_usec);
  
  hdr->format = htonl(encode_type);
  hdr->size = htonl(datalen);
  hdr->width = htonl(new_xsize);
  hdr->height = htonl(new_ysize);
  hdr->depth = htonl(req_val.attr.val[QOS_COLOR]);
  
  /*
  ** iov
  **  #0  header
  **  #1  body
  */
  iov[0].iov_base = (char *)hdr;
  iov[0].iov_len = sizeof(header);

  /* wait a tick. I don't know why :-p */
#ifdef USE_WAITTIME
  waittime.tv_sec = 0;
  waittime.tv_usec = 30000;   /* 30 ms */
#endif

  /* write mask check */
  for(fd=0; fd<=maxwritefds; fd++) {
    if (! FD_ISSET(fd, &writemask))
      continue;
#ifdef SUPPORT_MULTIRATE
    /* not implemented yet */
#endif

    hdr->offset = 0;
    rest = datalen;

    sendptr = new_dataPtr;
    send_count = 0;

    while (rest > 0) {
      iov[1].iov_base = sendptr;
      if (rest >= sentry_max_datalen)
	iov[1].iov_len = sentry_max_datalen;
      else 
	iov[1].iov_len = rest;

    try_write:
      nwrite = writev(fd, iov, 2);
      if (nrwite < 0) {
	fprintf(stderr, "send_image: writev: %s\n", sys_errlist[errno]);

	if (errno==EINTR)
	  goto try_write;
	goto fail;
      }

      send_datalen = nwrite - sizeof(header);
      hdr->offset += send_datalen;
      sendptr = &sendptr[send_datalen];
      rest -= send_datalen;

    } /* while for write-loop */

    fail:;
  } /* for-loop */

#else /* NOT IOVEC */
  fprintf(stderr, "Now, only USE_IOVEC version is supported.\n");
  exit (1);
#endif /* USE_IOVEC */

  release_frame();

  /*
   * for statistics information
   */
  show_stat();
}

/*
 * grab image (ysize offset is only supported.) 
 */
int 
grab_image(char **data, int *size, int *x, int *y, int gx, int gy, int xoff, int yoff)
{
  int color_depth;
  
  switch(req_val.attr.val[QOS_COLOR]) {
  case FULL_COLOR:
    color_depth = 4;
    break;
  case MONOCHROME:
  default:
    color_depth = 1;
    break;
  }

  *data = &(*data)[(*x)*yoff*color_depth];
  (*y) = gy;
  (*size) = gx * gy * color_depth;

  return 0;
}

#ifdef ALIVE
/*
 * plot alive mark in top line
 */
void
set_alive_mark(void)
{
  int i, color_depth;
  static int alive_pos = 0;

  switch(req_val.attr.val[QOS_COLOR]) {
  case FULL_COLOR:
    color_depth = 4;
    break;
  case MONOCHROME:
  default:
    color_depth = 1;
    break;
  }

  for(i=0; i<color_depth*2; i++) {
    dataPtr[alive_pos+i] = 0xff;
  }

  if (alive_pos+i-1 < xsize*color_depth)
    alive_pos += i;
  else
    alive_pos = 0;
}
#endif /* ALIVE */

/*
 * show send_image statistics information 
 */
void show_stat() 
{
  static int FrameCount = 0;
  static int TotalCount = -1;
  static struct timeval start_tv = {0, 0};
  struct timeval tv;
  
  if (start_tv.tv_sec == 0) 
    gettimeofday(&start_tv, NULL);
  gettimeofday(&tv, NULL);

  if (++TotalCount >= 1000) {
    if (tv.tv_usec -= start_tv.tv_usec) {
      tv.tv_sec -= 1;
      tv.tv_usec += 1000000;
    }
    tv.tv_sec -= start_tv.tv_sec;

    fprintf(stdout, "Total: frame# %d, uptime: %d.%06d s, rate# %.2f fps\n",
	    FrameCount, tv.tv_sec, tv.tv_usec, 
	    (((double)FrameCount)*1000000 / (tv.tv_sec*1000000+tv.tv_usec)));
    TotalCount = 0;
  }
  FrameCount ++;
}
